var Structr = require('structr'),
	Buffer = require('buffer').Buffer;

if (typeof(a3d) == 'undefined') {
	/** @namespace */ a3d = {};
}

a3d.Endian = {
	  BIG: 0
	, LITTLE: 1
};




//taken from http://www.adamia.com/blog/high-performance-javascript-port-of-actionscript-byteArray and tweaked a bit
var ByteArray = Structr({
	  data: ''
	, length: 0
	, pos: 0
	, pow: Math.pow
	, endian: a3d.Endian.BIG
	, TWOeN23: Math.pow(2, -23)
	, TWOeN52: Math.pow(2, -52)

	/** @constructor */
	, __construct: function(data, endian) {
		// data = (data || new Buffer(''));
		this.data = data;
		this.endian = endian = endian || a3d.Endian.BIG;
		this.length = data ? data.length : 0;

		// Cache the function pointers based on endianness.
		// This avoids doing an if-statement in every function call.
		var funcExt = (endian == a3d.Endian.BIG) ? 'BE' : 'LE';
		var funcs = ['readInt32', 'readInt16', 'readUInt32', 'readUInt16', 'readFloat32', 'readFloat64'];
		for (var func in funcs) {
			this[funcs[func]] = this[funcs[func] + funcExt];
		}

		// Add redundant members that match actionscript for compatibility
		var funcMap = {readUnsignedByte: 'readByte', readUnsignedInt: 'readUInt32',
			readFloat: 'readFloat32', readDouble: 'readFloat64', readShort: 'readInt16',
			readBoolean: 'readBool', readInt: 'readInt32'};
		for (var func in funcMap) {
			this[func] = this[funcMap[func]];
		}
	}
	, readUTFBytes: function(n){
		
		return this.data.toString('utf8',this.pos,this.pos+=n);
		
		// return this.data.toString('utf8',this.pos,(this.pos+=n));
	},
	peek:function(n,offset)
	{
		var p = this.pos;
		this.pos += (offset || 0);
		var buffer = this.readUTFBytes(n);
		this.pos = p;
		return buffer;
	}
	,clear:function()
	{
		this.data = new Buffer();
	}
	,readBytes: function(target,offset,length){
		target.data = this.data.slice(this.pos,this.pos+=length);
	}

	, readByte: function() {
		return (this.data[this.pos++] & 0xFF);
	}

	, readBool: function() {
		return (this.data[this.pos++] & 0xFF) ? true : false;
	}

	, readUInt32BE: function() {
		var data = this.data, pos = (this.pos += 4) - 4;
		return	((data[pos]   & 0xFF) << 24) |
				((data[++pos] & 0xFF) << 16) |
				((data[++pos] & 0xFF) << 8) |
				 (data[++pos] & 0xFF);
	}
	, readInt32BE: function() {
		var data = this.data, pos = (this.pos += 4) - 4;
		var x =	((data[pos]   & 0xFF) << 24) |
				((data[++pos] & 0xFF) << 16) |
				((data[++pos] & 0xFF) << 8) |
				 (data[++pos] & 0xFF);
		return (x >= 2147483648) ? x - 4294967296 : x;
	}

	, readUInt16BE: function() {
		var data = this.data, pos = (this.pos += 2) - 2;
		return	((data[pos]   & 0xFF) << 8) |
				 (data[++pos] & 0xFF);
	}
	, readInt16BE: function() {
		var data = this.data, pos = (this.pos += 2) - 2;
		var x =	((data[pos]   & 0xFF) << 8) |
				 (data[++pos] & 0xFF);
		return (x >= 32768) ? x - 65536 : x;
	}

	, readFloat32BE: function() {
		var data = this.data, pos = (this.pos += 4) - 4;
		var b1 = data[pos] & 0xFF,
			b2 = data[++pos] & 0xFF,
			b3 = data[++pos] & 0xFF,
			b4 = data[++pos] & 0xFF;
		var sign = 1 - ((b1 >> 7) << 1);                   // sign = bit 0
		var exp = (((b1 << 1) & 0xFF) | (b2 >> 7)) - 127;  // exponent = bits 1..8
		var sig = ((b2 & 0x7F) << 16) | (b3 << 8) | b4;    // significand = bits 9..31
		if (sig == 0 && exp == -127)
			return 0.0;
		return sign*(1 + this.TWOeN23*sig)*this.pow(2, exp);
	}

	, readFloat64BE: function() {
		var data = this.data, pos = (this.pos += 8) - 8;
		var b1 = data[pos] & 0xFF,
			b2 = data[++pos] & 0xFF,
			b3 = data[++pos] & 0xFF,
			b4 = data[++pos] & 0xFF,
			b5 = data[++pos] & 0xFF,
			b6 = data[++pos] & 0xFF,
			b7 = data[++pos] & 0xFF,
			b8 = data[++pos] & 0xFF;
		var sign = 1 - ((b1 >> 7) << 1);									// sign = bit 0
		var exp = (((b1 << 4) & 0x7FF) | (b2 >> 4)) - 1023;					// exponent = bits 1..11

		// This crazy toString() stuff works around the fact that js ints are
		// only 32 bits and signed, giving us 31 bits to work with
		var sig = (((b2 & 0xF) << 16) | (b3 << 8) | b4).toString(2) +
			((b5 >> 7) ? '1' : '0') +
			(((b5&0x7F) << 24) | (b6 << 16) | (b7 << 8) | b8).toString(2);	// significand = bits 12..63

		sig = parseInt(sig, 2);
		if (sig == 0 && exp == -1023)
			return 0.0;
		return sign*(1.0 + this.TWOeN52*sig)*this.pow(2, exp);
	}

	, readUInt32LE: function() {
		var data = this.data, pos = (this.pos += 4);
		return	((data[--pos]   & 0xFF) << 24) |
				((data[--pos] & 0xFF) << 16) |
				((data[--pos] & 0xFF) << 8) |
				 (data[--pos] & 0xFF);
	}
	, readInt32LE: function() {
		var data = this.data, pos = (this.pos += 4);
		
		var x =	((data[--pos]   & 0xFF) << 24) |
				((data[--pos] & 0xFF) << 16) |
				((data[--pos] & 0xFF) << 8) |
				 (data[--pos] & 0xFF);
		return (x >= 2147483648) ? x - 4294967296 : x;
	}

	, readUInt16LE: function() {
		var data = this.data, pos = (this.pos += 2);
		return	((data[--pos]   & 0xFF) << 8) |
				 (data[--pos] & 0xFF);
	}
	, readInt16LE: function() {
		var data = this.data, pos = (this.pos += 2);
		var x =	((data[--pos]   & 0xFF) << 8) |
				 (data[--pos] & 0xFF);
		return (x >= 32768) ? x - 65536 : x;
	}

	, readFloat32LE: function() {
		var data = this.data, pos = (this.pos += 4);
		var b1 = data[--pos] & 0xFF,
			b2 = data[--pos] & 0xFF,
			b3 = data[--pos] & 0xFF,
			b4 = data[--pos] & 0xFF;
		var sign = 1 - ((b1 >> 7) << 1);                   // sign = bit 0
		var exp = (((b1 << 1) & 0xFF) | (b2 >> 7)) - 127;  // exponent = bits 1..8
		var sig = ((b2 & 0x7F) << 16) | (b3 << 8) | b4;    // significand = bits 9..31
		if (sig == 0 && exp == -127)
			return 0.0;
		return sign*(1 + this.TWOeN23*sig)*this.pow(2, exp);
	}

	, readFloat64LE: function() {
		var data = this.data, pos = (this.pos += 8);
		var b1 = data[--pos] & 0xFF,
			b2 = data[--pos] & 0xFF,
			b3 = data[--pos] & 0xFF,
			b4 = data[--pos] & 0xFF,
			b5 = data[--pos] & 0xFF,
			b6 = data[--pos] & 0xFF,
			b7 = data[--pos] & 0xFF,
			b8 = data[--pos] & 0xFF;
		var sign = 1 - ((b1 >> 7) << 1);									// sign = bit 0
		var exp = (((b1 << 4) & 0x7FF) | (b2 >> 4)) - 1023;					// exponent = bits 1..11

		// This crazy toString() stuff works around the fact that js ints are
		// only 32 bits and signed, giving us 31 bits to work with
		var sig = (((b2 & 0xF) << 16) | (b3 << 8) | b4).toString(2) +
			((b5 >> 7) ? '1' : '0') +
			(((b5&0x7F) << 24) | (b6 << 16) | (b7 << 8) | b8).toString(2);	// significand = bits 12..63

		sig = parseInt(sig, 2);
		if (sig == 0 && exp == -1023)
			return 0.0;
		return sign*(1.0 + this.TWOeN52*sig)*this.pow(2, exp);
	}
});




exports.ByteArray = ByteArray;
