var Mongo = require('sk/node/db/mongo').Mongo,
    db = new Mongo('errorTypes');
    
var c_list = db.collection('list');

//list ofthe cached errors
var errorCodes = {};


c_list.find({},function(err,cursor)
{
  if(err) return;
  
  cursor.each(function(err,result)
  {
    // stack.each(function(result)
    
    if(!result)
      return;
      
      var prefix = result.prefix;
      var message = result.message;
      var code= result.code;
      
      
      //initialize the error codes stack
      if(!errorCodes[prefix]) errorCodes[prefix] = {};
      
      //the code is always the count that gets incremented
      errorCodes[prefix].$count = code;
      
      //cache the message so we don't hit the database each time we generate an error
      errorCodes[prefix][message] = code;
  });  
  

});


var ErrorFactory = function(prefix)
{
  prefix = prefix || 'E';
  
  //initialize the error code count if the type doesn't exist
  if(!errorCodes[prefix]) errorCodes[prefix] = {$count:0};
  var code;

  this.get = function()
  {
    var message = arguments[0];
    delete arguments[0];
    
    //get the error code from the cached list. this ONLY happens once. 
    if(!(code = errorCodes[prefix][message]))
    {
      //cache the error codes in memory for later use
      errorCodes[prefix][message] = code = errorCodes[prefix].$count = errorCodes[prefix].$count+1;
      
      //insert the error code in the database so it never changes
      c_list.insert({prefix:prefix,message:message,code:code},function(err,result)
      {                       
      });
    }  
    
    for(var i in arguments)
      message = message.replace(":?",arguments[i]);
    
    
    //return the error with the associated code
    return {error:{type:prefix+code,message:message}}
  }
}



exports.ErrorFactory = ErrorFactory;