var EventEmitter = require('events').EventEmitter,
Queue      = require('sk/core/queue').Queue,
utils      = require('sk/node/utils'),
sys = require('sys'),
util = require('util'),
Structr = require('structr'),
compress = require('compress');	


var AbstractViewController = {                     
	evaluated:false,
	children:null, 
	viewName:null,
	data:null,
	parent:null,
	viewInfo:null,  
	request:null,
	fileExtension:null,
	defaultRootView:null,
	defaultViewPrefix:null,     

	//the prepare callback executed before anything is displayed. This is useful for adding any additional
	//view controllers such as header items (javascript)
	prepareCallback:null,
	prepared:false,

	on:function(type,callback)
	{
		this.emitter().on(type,callback);
	},

	display: function(request)
	{            
		if(this.evaluated)
		return;

		this.request = request;
		this.evaluated = true;      


		this.displayView(request);           
	},
	prepare: function()
	{        
		if(this.prepared) 
		return;


		var children = this.children();

		for(var i in children)
		{
			stack = children[i];
			for(j in stack)
			{
				stack[j].prepare();
			}
		}


		this.prepared = true;

		if(this.preparedCallback)
		{
			this.preparedCallback();
		}
	}, 

	reset: function()
	{
		this.evaluated = false;  
		this.prepared = false;
		return this;
	},       

	getRootViewDir: function ()
	{
		return this.viewInfo.root || this.defaultRootView;
	},

	getViewDir: function ()
	{
		return this.getRootViewDir() + (this.viewInfo.prefix || this.defaultViewPrefix);
	}, 

	emitter: function ()
	{
		if(!this._emitter)
		{
			this._emitter = new EventEmitter();
		}                                      

		return this._emitter;
	},               


	queue: function ()
	{
		if(!this._queue)
		{                            
			this._queue = new Queue();
			this._queue.on('complete',function()
			{
				//done... 
			});
		}                 

		return this._queue;
	},   

	children: function ()
	{
		if(!this._children)
		{
			this._children = [];
		}                       

		return this._children;
	},                                         

	'abstract displayView': function (request) { },

	getRoot: function ()
	{
		var p = this;

		while(p.parent)
		{
			p = p.parent;
		}                

		return p;
	}, 

	onChunk: function (chunk)
	{
		self.send(chunk);
	},       

	addChild: function (child, type)
	{              
		type = type || 'body'; 

		var children = this.children();

		if(!children[type])
		{
			children[type] = [];
		}          


		child.parent = this;
		children[type].push(child);        

		if(this.prepared)
		{
			child.prepare();
		}
	}, 

	getChildAt: function (index, type)
	{            
		var children = this.children();

		if(!children[type])
		return null;

		return children[type][index];
	},                                

	getChildren: function (type)
	{
		if(!children[type])
		return [];

		return children[type];
	},                         

	addRootChild: function (child, type)
	{
		self.getRoot().addChild(child,type);
	},                                       

	displayChildAt: function (index, type)
	{
		var cvc = self.getChildAt(index,type);

		if(!cvc)
		return;

		cvc.display()
	},

	displayChildren: function (type)
	{           
		var children = this.children();

		if(children[type])
		{       
			var stack = children[type];

			for(var i in stack)
			{                      
				this.displayChild(stack[i]); 
			}
		}

		return this.queue();
	}, 


	displayChild: function (child)
	{   
		var queue = this.queue();  

		var self = this;   


		queue.add( function (queue)
		{    
			child.on('body',function(chunk)
			{                                                                                                                                                                                                                  
				self.send(chunk);
			});          

			child.on('complete',function(chunk)
			{
				self.send(chunk);    
				queue.next();
			});

			child.reset().display(self.request);
		});

		queue.start();
	},

	send: function (data)
	{                                 
		this.emitter().emit('body',data);    
	},        

	complete: function (data)
	{  
		this.emitter().emit('complete',data || '');
	}


} 

AbstractViewController = Structr(AbstractViewController);
 

var EchoViewController = AbstractViewController.extend({
  displayView: function (request)
  {
    this.end(this.data);
  }
}); 

var gzip = new compress.Gzip();


var JSONViewController = AbstractViewController.extend({
	'decorate __construct': function()
	{
		// this.gzip = new compress.Gzip();
		// this.gzip.init();
	},
	displayView: function (request)
	{	
		var resultString;
		
		if(request)
		{
			request.headers['Content-Type'] = 'application/json';

			var resultString = request.data.pretty != undefined ? util.inspect(this.data,false,null) : JSON.stringify(this.data);
			
			var client = request.client;
			
			//jsonp. used primarily for browsers that don't allow XSS such as fucking IE ugh >.>/
			if(request.data.callback)
			{
				resultString = request.data.callback + '(' + resultString + ');';
			}
			
			//should this be here???? Gzip is broken.
			/*if(client.headers['accept-encoding'] && client.headers['accept-encoding'].indexOf('gzip') > -1)
			{
				
				this.gzip.init();
				resultString = this.gzip.deflate(new Buffer('my name is craig'));
				request.headers['Content-Encoding'] = 'gzip';
			}*/
		}
		else
		{
			resultString = JSON.stringify(this.data);
		}
		

		this.complete(resultString);
	}
})
     

function ViewFactory()
{     
	var self = this,
	viewTemplates = {};
	this.viewClasses = {};


	this.getViewController = function(viewInfo,viewName,fileExtension,data)
	{                   
		data = data || {};                      
		fileExtension = fileExtension || 'html';

		//the view name must always end with the template type. e.g: default.tpl.ejs
		var viewType = viewName.split('.').pop();


		var viewClass = self.viewClasses[viewType];

		if(viewClass)
		{
			var c = new viewClass();  
			c.viewName      = viewName;
			c.data          = data;   
			c.viewInfo      = viewInfo;    
			c.fileExtension = fileExtension;   
			c.defaultViewPrefix = '/';
		}        
		else
		{                 
			console.warning("Warning: no view controller \""+viewType+"\" exists.");
			return false;
		} 

		c.reset();

		return c;
	}

	this.addViewClass = function(clazz,type)
	{
		this.viewClasses[type] = clazz;
	}

	this.registerTemplate = function(viewInfo,name)
	{
		viewTemplates[name] = viewInfo;
	}

	this.display = function(request, viewName, data)
	{
		self.template(viewName, data, {
			send:request.send,
			end:request.end
		},request);
	}
	
	this.template = function(viewName, data, callbacks, request)
	{
		if(!callbacks)
		{
			request = callbacks;
			callbacks = data;
			data = undefined;
		}
		
		if(callbacks instanceof Function)
		{
			var callback = callbacks,
				buffer = [];
			
			callbacks = {
				send: function(chunk)
				{
					buffer.push(chunk);
				}
				,end: function(chunk)
				{
					if(chunk)
					{
						buffer.push(chunk);
					}
					
					callback(buffer.join(''));
				}
			}
		}
		
		var viewParts = viewName.split('.');
		
		fileExtension = request ? request.fileExtension : viewParts.pop();
		
		viewName = viewParts.join('.');
		
		
		var controller = self.getViewController(request ? request.currentRoute.viewInfo : {root:''}, viewName, fileExtension, data);
		
		if(!controller)
		{
			return callbacks.end(false);
		}
		
		controller.on('body',callbacks.send)
		controller.on('complete',callbacks.end);
		
		return controller.display(request);
	}
	
} 

      
exports.ViewFactory = exports.View = new ViewFactory();

exports.ViewFactory.addViewClass(JSONViewController,'json');  
exports.ViewFactory.addViewClass(EchoViewController,'echo');               
exports.AbstractViewController = AbstractViewController;   
exports.EchoViewController = EchoViewController;

                          

